from tqdm.auto import tqdm

from ..utils import logger, constants
from ..services import *

log = logger.setup_logger()


def upload_model(
    folder: str,
    repo_id: str,
    model_name: str,
):
    # OpeniFileUpload
    local_dir, targe_files = get_upload_folder_files(folder)
    if targe_files == []:
        msg = f"❌ {local_dir} 未找到任何本地文件"
        log.error(msg)
        raise ValueError(msg)

    # OpeniRepo, check repo_id, access
    openi_repo = OpeniRepo(repo_id=repo_id)

    # OpeniModel
    openi_model = OpeniModel(
        model_name=model_name,
        repo_id=openi_repo.repo_id,
    )
    if openi_model.can_operate is False:
        msg = f"❌ `{openi_repo.current_user}` 无此模型上传权限，模型名称 `{openi_model.model_name}`, 仓库名称 `{openi_model.repo_id}`"
        log.error(msg)
        raise ValueError(msg)
    if openi_model.model_type == 0:
        msg = f"❌ `{openi_model.model_name}` 为线上模型，无法上传本地文件"
        log.error(msg)
        raise ValueError(msg)

    # openifiles
    tiltle_pbar = tqdm(
        total=len(targe_files),
        leave=True,
        bar_format="{desc}({n_fmt}/{total_fmt}) |{bar}| [{elapsed}]",
        desc=f"Calculating file md5… ",
        position=0,
        # dynamic_ncols=True,
    )
    openifiles = list()
    for file in targe_files:
        openi_file = OpeniFileUpload(
            local_file=file,
            repo_id=openi_repo.repo_id,
            upload_type=constants.FILE.DEFAULT_UPLOAD_TYPE,
            dataset_or_model_id=openi_model.model_id,
            dataset_or_model_name=openi_model.model_name,
            target_type="model",
        )
        tiltle_pbar.update(1)
        openifiles.append(openi_file)
    tiltle_pbar.close()

    # # Display pbar
    display_progress_bar(openifiles)

    # # Upload
    for openi_file in openifiles:
        openi_file.upload_with_tqdm()



import os
import json
from getpass import getpass
from pathlib import Path

from .services import get_login_user
from .utils import logger, constants
from .utils.logger import *

log = logger.setup_logger()


def login(token: str = None, endpoint: str = constants.API.DEFAULT_ENDPOINT) -> None:
    print(
        """\n
             ██████╗   ██████╗  ███████╗  ███╗   ██╗  ██████╗
            ██╔═══██╗  ██╔══██╗ ██╔════╝  ████╗  ██║    ██╔═╝
            ██║   ██║  ██████╔╝ █████╗    ██╔██╗ ██║    ██║
            ██║   ██║  ██╔═══╝  ██╔══╝    ██║╚██╗██║    ██║
            ╚██████╔╝  ██║      ███████╗  ██║ ╚████║  ██████╗
             ╚═════╝   ╚═╝      ╚══════╝  ╚═╝  ╚═══╝  ╚═════╝\n
         """
    )
    endpoint = endpoint[:-1] if endpoint[-1] == "/" else endpoint

    if token is None:
        print(f"点击链接获取令牌并复制粘贴到下列输入栏 {endpoint}/user/settings/applications \n")
        print(
            f"[WARNING] 若本机已存在登录令牌，本次输入的令牌会将其覆盖 \n"
            "          若粘贴时切换了本窗口，请先按 退格键⇦ 删除多余空格\n"
        )
        token = getpass(prompt="  🔒 token: ")

    # _login(token=token, endpoint=endpoint)

    try:
        user_info, api_endpoint = get_login_user(token=token, endpoint=endpoint)
        username = user_info["username"]
        msg = (
            f"\n  Your token was saved to `{constants.PATH.TOKEN_PATH}`\n"
            f"  Successfully logged in as `{username}` @{api_endpoint}!\n"
        )
        Path(constants.PATH.TOKEN_PATH).write_text(
            json.dumps({"endpoint": endpoint, "token": token})
        )
        log.info(msg)
        print(msg)
    except:
        # os.remove(constants.PATH.TOKEN_PATH)
        msg = "\n  ❌ login failed: invalid token!\n"
        log.error(msg)
        # raise ValueError(msg)
        print(msg)


def whoami() -> None:
    try:
        user_info, endpoint = get_login_user()
        username = user_info["username"]
        msg = f"\n`{username}` @{endpoint}\n"
        log.info(msg)
        print(msg)
    except:
        msg = f"\n无法获取当前用户信息, 请检查网络或 {constants.PATH.TOKEN_PATH} \n"
        log.info(msg)
        print(msg)


def logout() -> None:
    try:
        user_info, _ = get_login_user()
        username = user_info["username"]
        os.remove(constants.PATH.TOKEN_PATH)
        msg = f"\n`{username}` successfully logged out.\n"
        log.info(msg)
        print(msg)
    except:
        msg = "\nCurrently not logged in.\n"
        log.info(msg)
        print(msg)



import os

from ..utils import logger, constants
from ..utils.logger import *
from ..services import *

log = logger.setup_logger()


def download_model(
    repo_id: str,
    model_name: str,
    save_path: str = constants.PATH.MODEL_SAVE_PATH,
):
    openi_repo = OpeniRepo(repo_id=repo_id)
    if openi_repo.access not in ["read", "write"]:
        msg = f"`{repo_id}` 无权限操作此仓库"
        log.error(msg)
        raise ValueError(msg)

    openi_model = OpeniModel(
        model_name=model_name,
        repo_id=openi_repo.repo_id,
    )
    if openi_model.model_files == []:
        msg = f"❌ `{model_name}` 未找到文件"
        log.error(msg)
        raise ValueError(msg)
    if openi_model.can_download is False:
        msg = f"❌ 本机登录用户无此模型下载权限，模型名称 `{openi_model.model_name}`, 仓库名称 `{openi_model.repo_id}`"
        log.error(msg)
        raise ValueError(msg)

    save_path = os.path.expanduser(save_path)
    local_dir = os.path.join(save_path, f"{model_name}")
    if not os.path.exists(local_dir):
        os.makedirs(local_dir, exist_ok=True)

    openifiles = list()
    for targe_file in openi_model.model_files:
        openi_file = OpeniFileDownload(
            filename=targe_file["fileName"],
            size=targe_file["size"],
            local_dir=local_dir,
            repo_id=openi_repo.repo_id,
            dataset_or_model_id=openi_model.model_id,
            dataset_or_model_name=openi_model.model_name,
            target_type="model",
        )
        openifiles.append(openi_file)

    display_progress_bar(openifiles)
    for openi_file in openifiles:
        openi_file.download_with_tqdm()



from tqdm.auto import tqdm

from ..utils import logger, constants

log = logger.setup_logger()


def display_progress_bar(file_list: list):
    """
    display progress bar for files
    :param
        file_list: a list of OpeniFile objects contains "filename" and "size"
    """
    widths = [len(openi_file.filename) for openi_file in file_list]
    max_width = min(int(sum(widths) / len(widths)), 25)

    for index, openi_file in enumerate(file_list):
        filename, size = openi_file.filename, openi_file.size

        if len(filename) > max_width:
            title = "…" + filename[-1 * max_width :]
        else:
            title = filename.rjust(max_width + 1)

        desc = f"Waiting({title}): "
        if openi_file.target_type == "dataset":
            cluster = constants.FILE.UPLOAD_TYPE[openi_file.upload_type]
            desc = f"Waiting({title})({cluster}): "

        bar_format = (
            "{desc}{percentage:3.0f}%|{bar}| "
            "{n_fmt}{unit}/{total_fmt}{unit} "
            "[{elapsed}<{remaining}, "
            "{rate_fmt}{postfix}]"
        )
        pbar = tqdm(
            total=size,
            leave=True,
            unit="B",
            unit_scale=True,
            unit_divisor=1024,
            bar_format=bar_format,
            desc=desc,
            position=index + 1,
            # dynamic_ncols=True,
        )
        openi_file.pbar = pbar



# from .apis import *  # OpeniAPI
# from .utils import *  # file_utils, logger, constants
# from .services import *

from .login import login, whoami, logout
from .dataset import upload_file, download_file
from .model import upload_model, download_model
